@@ -15,17 +15,18 @@ $required_php_exts = array(
     'XML'       => 'xml',
     'JSON'      => 'json',
     'PDO'       => 'PDO',
+    'Multibyte' => 'mbstring',
+    'OpenSSL'   => 'openssl',
 );
 
 $optional_php_exts = array(
     'FileInfo'  => 'fileinfo',
     'Libiconv'  => 'iconv',
-    'Multibyte' => 'mbstring',
-    'OpenSSL'   => 'openssl',
-    'Mcrypt'    => 'mcrypt',
     'Intl'      => 'intl',
     'Exif'      => 'exif',
     'LDAP'      => 'ldap',
+    'GD'        => 'gd',
+    'Imagick'   => 'imagick',
 );
 
 $required_libs = array(
@@ -34,7 +35,6 @@ $required_libs = array(
     'Net_SMTP'  => 'pear.php.net',
     'Net_IDNA2' => 'pear.php.net',
     'Mail_mime' => 'pear.php.net',
-    'Mail_mimeDecode' => 'pear.php.net',
 );
 
 $optional_libs = array(
@@ -46,15 +46,12 @@ $ini_checks = array(
     'session.auto_start'            => 0,
     'mbstring.func_overload'        => 0,
     'suhosin.session.encrypt'       => 0,
-    'magic_quotes_runtime'          => 0,
-    'magic_quotes_sybase'           => 0,
 );
 
 $optional_checks = array(
     // required for utils/modcss.inc, should we require this?
     'allow_url_fopen'  => 1,
     'date.timezone'    => '-VALID-',
-    'register_globals' => 0, // #1489157
 );
 
 $source_urls = array(
@@ -64,7 +61,6 @@ $source_urls = array(
     'FileInfo'  => 'http://www.php.net/manual/en/book.fileinfo.php',
     'Libiconv'  => 'http://www.php.net/manual/en/book.iconv.php',
     'Multibyte' => 'http://www.php.net/manual/en/book.mbstring.php',
-    'Mcrypt'    => 'http://www.php.net/manual/en/book.mcrypt.php',
     'OpenSSL'   => 'http://www.php.net/manual/en/book.openssl.php',
     'JSON'      => 'http://www.php.net/manual/en/book.json.php',
     'DOM'       => 'http://www.php.net/manual/en/book.dom.php',
@@ -73,6 +69,8 @@ $source_urls = array(
     'oci8'      => 'http://www.php.net/manual/en/book.oci8.php',
     'PDO'       => 'http://www.php.net/manual/en/book.pdo.php',
     'LDAP'      => 'http://www.php.net/manual/en/book.ldap.php',
+    'GD'        => 'http://www.php.net/manual/en/book.image.php',
+    'Imagick'   => 'http://www.php.net/manual/en/book.imagick.php',
     'pdo_mysql'   => 'http://www.php.net/manual/en/ref.pdo-mysql.php',
     'pdo_pgsql'   => 'http://www.php.net/manual/en/ref.pdo-pgsql.php',
     'pdo_sqlite'  => 'http://www.php.net/manual/en/ref.pdo-sqlite.php',
@@ -82,9 +80,8 @@ $source_urls = array(
     'PEAR'      => 'http://pear.php.net',
     'Net_SMTP'  => 'http://pear.php.net/package/Net_SMTP',
     'Mail_mime' => 'http://pear.php.net/package/Mail_mime',
-    'Mail_mimeDecode' => 'http://pear.php.net/package/Mail_mimeDecode',
     'Net_IDNA2' => 'http://pear.php.net/package/Net_IDNA2',
-    'Net_LDAP3' => 'http://git.kolab.org/pear/Net_LDAP3',
+    'Net_LDAP3' => 'https://git.kolab.org/diffusion/PNL',
 );
 
 echo '<input type="hidden" name="_step" value="' . ($RCI->configured ? 3 : 2) . '" />';
@@ -93,7 +90,7 @@ echo '<input type="hidden" name="_step" value="' . ($RCI->configured ? 3 : 2) .
 <h3>Checking PHP version</h3>
 <?php
 
-define('MIN_PHP_VERSION', '5.3.7');
+define('MIN_PHP_VERSION', '5.4.0');
 if (version_compare(PHP_VERSION, MIN_PHP_VERSION, '>=')) {
     $RCI->pass('Version', 'PHP ' . PHP_VERSION . ' detected');
 } else {
