@@ -23,6 +23,7 @@ window.rcmail && rcmail.addEventListener('init', function(evt) {
             rcmail.register_command('search', function(props) {return rcmail.enigma_search(props); }, true);
             rcmail.register_command('reset-search', function(props) {return rcmail.enigma_search_reset(props); }, true);
             rcmail.register_command('plugin.enigma-import', function() { rcmail.enigma_import(); }, true);
+            rcmail.register_command('plugin.enigma-import-search', function() { rcmail.enigma_import_search(); }, true);
             rcmail.register_command('plugin.enigma-key-export', function() { rcmail.enigma_export(); });
             rcmail.register_command('plugin.enigma-key-export-selected', function() { rcmail.enigma_export(true); });
             rcmail.register_command('plugin.enigma-key-import', function() { rcmail.enigma_key_import(); }, true);
@@ -33,6 +34,19 @@ window.rcmail && rcmail.addEventListener('init', function(evt) {
             rcmail.addEventListener('responseafterplugin.enigmakeys', function() {
                 rcmail.enable_command('plugin.enigma-key-export', rcmail.env.rowcount > 0);
             });
+
+            if (rcmail.gui_objects.importform) {
+                // make sure Enter key in search input starts searching
+                // instead of submitting the form
+                $('#rcmimportsearch').keydown(function(e) {
+                    if (e.which == 13) {
+                        rcmail.enigma_import_search();
+                        return false;
+                    }
+                });
+
+                $('input[type="button"]:first').focus();
+            }
         }
     }
     else if (rcmail.env.task == 'mail') {
@@ -44,6 +58,8 @@ window.rcmail && rcmail.addEventListener('init', function(evt) {
                 // don't close the menu on mouse click inside
                 e.stopPropagation();
             });
+
+            $('a.button.enigma').prop('tabindex', $('#messagetoolbar > a:first').prop('tabindex'));
         }
 
         $.each(['encrypt', 'sign'], function() {
@@ -77,12 +93,15 @@ rcube_webmail.prototype.enigma_key_create = function()
 // Generate key(s) and submit them
 rcube_webmail.prototype.enigma_key_create_save = function()
 {
-    var options, lock,
-        user = $('#key-ident > option').filter(':selected').text(),
+    var options, lock, users = [],
         password = $('#key-pass').val(),
         confirm = $('#key-pass-confirm').val(),
         size = $('#key-size').val();
 
+    $('[name="identity[]"]:checked').each(function() {
+        users.push({name: $(this).data('name') || '', email: $(this).data('email')});
+    });
+
     // validate the form
     if (!password || !confirm)
         return alert(this.get_label('enigma.formerror'));
@@ -90,37 +109,33 @@ rcube_webmail.prototype.enigma_key_create_save = function()
     if (password != confirm)
         return alert(this.get_label('enigma.passwordsdiffer'));
 
-    if (user.match(/^<[^>]+>$/))
-        return alert(this.get_label('enigma.nonameident'));
+    if (!users.length)
+        return alert(this.get_label('enigma.noidentselected'));
 
     // generate keys
     // use OpenPGP.js if browser supports required features
-    if (window.openpgp && window.crypto && (window.crypto.getRandomValues || window.crypto.subtle)) {
+    if (window.openpgp && (window.msCrypto || (window.crypto && (window.crypto.getRandomValues || window.crypto.subtle)))) {
         lock = this.set_busy(true, 'enigma.keygenerating');
         options = {
             numBits: size,
-            userId: user,
+            userIds: users,
             passphrase: password
         };
 
-        openpgp.generateKeyPair(options).then(function(keypair) {
+        openpgp.generateKey(options).then(function(keypair) {
             // success
-            var post = {_a: 'import', _keys: keypair.privateKeyArmored};
+            var post = {_a: 'import', _keys: keypair.privateKeyArmored, _generated: 1,
+                _passwd: password, _keyid: keypair.key.primaryKey.fingerprint};
 
             // send request to server
             rcmail.http_post('plugin.enigmakeys', post, lock);
         }, function(error) {
             // failure
+            console.error(error);
             rcmail.set_busy(false, null, lock);
             rcmail.display_message(rcmail.get_label('enigma.keygenerateerror'), 'error');
         });
     }
-    // generate keys on the server
-    else if (rcmail.env.enigma_keygen_server) {
-        lock = this.set_busy(true, 'enigma.keygenerating');
-        options = {_a: 'generate', _user: user, _password: password, _size: size};
-        rcmail.http_post('plugin.enigmakeys', options, lock);
-    }
     else {
         rcmail.display_message(rcmail.get_label('enigma.keygennosupport'), 'error');
     }
@@ -150,18 +165,76 @@ rcube_webmail.prototype.enigma_delete = function()
 // Export key(s)
 rcube_webmail.prototype.enigma_export = function(selected)
 {
-    var keys = selected ? this.keys_list.get_selection().join(',') : '*';
+    var priv = false,
+        list = this.keys_list,
+        keys = selected ? list.get_selection().join(',') : '*',
+        args = {_keys: keys};
 
     if (!keys.length)
         return;
 
-    this.goto_url('plugin.enigmakeys', {_a: 'export', _keys: keys}, false, true);
+    // find out whether selected keys are private
+    if (keys == '*')
+        priv = true;
+    else
+        $.each(list.get_selection(), function() {
+            flags = $(list.rows[this].obj).data('flags');
+            if (flags && flags.indexOf('p') >= 0) {
+                priv = true;
+                return false;
+            }
+        });
+
+    // ask the user about including private key in the export
+    if (priv)
+        return this.show_popup_dialog(
+            this.get_label('enigma.keyexportprompt'),
+            this.get_label('enigma.exportkeys'),
+            [{
+                text: this.get_label('enigma.onlypubkeys'),
+                click: function(e) {
+                    rcmail.enigma_export_submit(args);
+                    $(this).remove();
+                }
+            },
+            {
+                text: this.get_label('enigma.withprivkeys'),
+                click: function(e) {
+                    args._priv = 1;
+                    rcmail.enigma_export_submit(args);
+                    $(this).remove();
+                }
+            }],
+            {width: 400}
+        );
+
+    this.enigma_export_submit(args);
+};
+
+// Sumbitting request for key(s) export
+// Done this way to handle password input
+rcube_webmail.prototype.enigma_export_submit = function(data)
+{
+    var id = 'keyexport-' + new Date().getTime(),
+        form = $('<form>').attr({target: id, method: 'post', style: 'display:none',
+            action: '?_action=plugin.enigmakeys&_task=settings&_a=export'}),
+        iframe = $('<iframe>').attr({name: id, style: 'display:none'})
+
+    form.append($('<input>').attr({name: '_token', value: this.env.request_token}));
+    $.each(data, function(i, v) {
+        form.append($('<input>').attr({name: i, value: v}));
+    });
+
+    iframe.appendTo(document.body);
+    form.appendTo(document.body).submit();
 };
 
 // Submit key(s) import form
 rcube_webmail.prototype.enigma_import = function()
 {
-    var form, file;
+    var form, file, lock,
+        id = 'keyexport-' + new Date().getTime(),
+        iframe = $('<iframe>').attr({name: id, style: 'display:none'});
 
     if (form = this.gui_objects.importform) {
         file = document.getElementById('rcmimportfile');
@@ -170,13 +243,26 @@ rcube_webmail.prototype.enigma_import = function()
             return;
         }
 
-        var lock = this.set_busy(true, 'importwait');
+        lock = this.set_busy(true, 'importwait');
+        iframe.appendTo(document.body);
+        $(form).attr({target: id, action: this.add_url(form.action, '_unlock', lock)})
+            .submit();
+    }
+};
 
-        form.action = this.add_url(form.action, '_unlock', lock);
-        form.submit();
+// Ssearch for key(s) for import
+rcube_webmail.prototype.enigma_import_search = function()
+{
+    var form, search;
 
-        this.lock_form(form, true);
-   }
+    if (form = this.gui_objects.importform) {
+        search = $('#rcmimportsearch').val();
+        if (!search) {
+            return;
+        }
+
+        this.enigma_find_publickey(search);
+    }
 };
 
 // list row selection handler
@@ -207,17 +293,19 @@ rcube_webmail.prototype.enigma_keylist_keypress = function(list)
 // load key frame
 rcube_webmail.prototype.enigma_loadframe = function(url)
 {
-    var frm, win;
+    var win;
 
-    if (this.env.contentframe && window.frames && (frm = window.frames[this.env.contentframe])) {
-        if (!url && (win = window.frames[this.env.contentframe])) {
+    if (win = this.get_frame_window(this.env.contentframe)) {
+        if (!url) {
             if (win.location && win.location.href.indexOf(this.env.blankpage) < 0)
                 win.location.href = this.env.blankpage;
+            if (this.env.frame_lock)
+                this.set_busy(false, null, this.env.frame_lock);
             return;
         }
 
         this.env.frame_lock = this.set_busy(true, 'loading');
-        frm.location.href = this.env.comm_path + '&_framed=1&' + url;
+        win.location.href = this.env.comm_path + '&_framed=1' + url;
     }
 };
 
@@ -228,10 +316,10 @@ rcube_webmail.prototype.enigma_search = function(props)
         props = this.gui_objects.qsearchbox.value;
 
     if (props || this.env.search_request) {
-        var params = {'_a': 'search', '_q': urlencode(props)},
+        var params = {'_a': 'search', '_q': props},
           lock = this.set_busy(true, 'searching');
 //        if (this.gui_objects.search_filter)
-  //          addurl += '&_filter=' + this.gui_objects.search_filter.value;
+//          addurl += '&_filter=' + this.gui_objects.search_filter.value;
         this.env.current_page = 1;
         this.enigma_loadframe();
         this.enigma_clear_list();
@@ -239,7 +327,7 @@ rcube_webmail.prototype.enigma_search = function(props)
     }
 
     return false;
-}
+};
 
 // Reset search filter and the list
 rcube_webmail.prototype.enigma_search_reset = function(props)
@@ -256,11 +344,14 @@ rcube_webmail.prototype.enigma_search_reset = function(props)
     }
 
     return false;
-}
+};
 
 // Keys/certs listing
-rcube_webmail.prototype.enigma_list = function(page)
+rcube_webmail.prototype.enigma_list = function(page, reset_frame)
 {
+    if (this.is_framed())
+        return parent.rcmail.enigma_list(page, reset_frame);
+
     var params = {'_a': 'list'},
       lock = this.set_busy(true, 'loading');
 
@@ -271,9 +362,9 @@ rcube_webmail.prototype.enigma_list = function(page)
     if (page)
         params._p = page;
 
-    this.enigma_clear_list();
+    this.enigma_clear_list(reset_frame);
     this.http_post('plugin.enigmakeys', params, lock);
-}
+};
 
 // Change list page
 rcube_webmail.prototype.enigma_list_page = function(page)
@@ -288,17 +379,19 @@ rcube_webmail.prototype.enigma_list_page = function(page)
         page = 1;
 
     this.enigma_list(page);
-}
+};
 
 // Remove list rows
-rcube_webmail.prototype.enigma_clear_list = function()
+rcube_webmail.prototype.enigma_clear_list = function(reset_frame)
 {
-    this.enigma_loadframe();
+    if (reset_frame !== false)
+        this.enigma_loadframe();
+
     if (this.keys_list)
         this.keys_list.clear(true);
 
     this.enable_command('plugin.enigma-key-delete', 'plugin.enigma-key-delete-selected', false);
-}
+};
 
 // Adds a row to the list
 rcube_webmail.prototype.enigma_add_list_row = function(r)
@@ -318,11 +411,12 @@ rcube_webmail.prototype.enigma_add_list_row = function(r)
 
     row.id = 'rcmrow' + r.id;
     row.className = css_class;
+    if (r.flags) $(row).data('flags', r.flags);
 
     col.innerHTML = r.name;
     row.appendChild(col);
     list.insert_row(row);
-}
+};
 
 
 /*********************************************************/
@@ -334,13 +428,13 @@ rcube_webmail.prototype.enigma_beforesend_handler = function(props)
 {
     this.env.last_action = 'send';
     this.enigma_compose_handler(props);
-}
+};
 
 rcube_webmail.prototype.enigma_beforesavedraft_handler = function(props)
 {
     this.env.last_action = 'savedraft';
     this.enigma_compose_handler(props);
-}
+};
 
 rcube_webmail.prototype.enigma_compose_handler = function(props)
 {
@@ -362,7 +456,7 @@ rcube_webmail.prototype.enigma_compose_handler = function(props)
     if (this.env.last_action == 'savedraft') {
         $('input[name="_enigma_sign"]', form).val(0);
     }
-}
+};
 
 // Import attached keys/certs file
 rcube_webmail.prototype.enigma_import_attachment = function(mime_id)
@@ -373,7 +467,7 @@ rcube_webmail.prototype.enigma_import_attachment = function(mime_id)
     this.http_post('plugin.enigmaimport', post, lock);
 
     return false;
-}
+};
 
 // password request popup
 rcube_webmail.prototype.enigma_password_request = function(data)
@@ -437,27 +531,47 @@ rcube_webmail.prototype.enigma_password_request = function(data)
         // this fixes bug when pressing Enter on "Save" button in the dialog
         parent.rcmail.message_list.blur();
     }
-}
+};
 
 // submit entered password
 rcube_webmail.prototype.enigma_password_submit = function(data)
 {
+    var lock, form;
+
     if (this.env.action == 'compose' && !data['compose-init']) {
         return this.enigma_password_compose_submit(data);
     }
+    else if (this.env.action == 'plugin.enigmakeys' && (form = this.gui_objects.importform)) {
+        if (!$('input[name="_keyid"]', form).length) {
+            $(form).append($('<input>').attr({type: 'hidden', name: '_keyid', value: data.key}))
+                .append($('<input>').attr({type: 'hidden', name: '_passwd', value: data.password}))
+        }
 
-    var lock = this.set_busy(true, 'loading');
+        return this.enigma_import();
+    }
 
-    // message preview
-    var form = $('<form>').attr({method: 'post', action: location.href, style: 'display:none'})
+    lock = data.nolock ? null : this.set_busy(true, 'loading');
+    form = $('<form>')
+        .attr({method: 'post', action: data.action || location.href, style: 'display:none'})
         .append($('<input>').attr({type: 'hidden', name: '_keyid', value: data.key}))
         .append($('<input>').attr({type: 'hidden', name: '_passwd', value: data.password}))
         .append($('<input>').attr({type: 'hidden', name: '_token', value: this.env.request_token}))
-        .append($('<input>').attr({type: 'hidden', name: '_unlock', value: lock}))
-        .appendTo(document.body);
+        .append($('<input>').attr({type: 'hidden', name: '_unlock', value: lock}));
 
-    form.submit();
-}
+    // Additional form fields for request parameters
+    $.each(data, function(i, v) {
+        if (i.indexOf('input') == 0)
+            form.append($('<input>').attr({type: 'hidden', name: i.substring(5), value: v}))
+    });
+
+    if (data.iframe) {
+        var name = 'enigma_frame_' + (new Date()).getTime(),
+            frame = $('<iframe>').attr({style: 'display:none', name: name}).appendTo(document.body);
+        form.attr('target', name);
+    }
+
+    form.appendTo(document.body).submit();
+};
 
 // submit entered password - in mail compose page
 rcube_webmail.prototype.enigma_password_compose_submit = function(data)
@@ -474,4 +588,39 @@ rcube_webmail.prototype.enigma_password_compose_submit = function(data)
     }
 
     this.submit_messageform(this.env.last_action == 'savedraft');
-}
+};
+
+// Display no-key error with key search button
+rcube_webmail.prototype.enigma_key_not_found = function(data)
+{
+    return this.show_popup_dialog(
+        data.text,
+        data.title,
+        [{
+            text: data.button,
+            click: function(e) {
+                $(this).remove();
+                rcmail.enigma_find_publickey(data.email);
+            }
+        }],
+        {width: 400, dialogClass: 'error'}
+    );
+};
+
+// Search for a public key on the key server
+rcube_webmail.prototype.enigma_find_publickey = function(email)
+{
+    this.mailvelope_search_pubkeys([email],
+        function(status) {},
+        function(key) {
+            var lock = rcmail.set_busy(true, 'enigma.importwait'),
+                post = {_a: 'import', _keys: key};
+
+            if (rcmail.env.action == 'plugin.enigmakeys')
+                post._refresh = 1;
+
+            // send request to server
+            rcmail.http_post('plugin.enigmakeys', post, lock);
+        }
+    );
+};
