@@ -3,7 +3,6 @@
 /**
  * Password Plugin for Roundcube
  *
- * @version @package_version@
  * @author Aleksander Machniak <alec@alec.pl>
  *
  * Copyright (C) 2005-2015, The Roundcube Dev Team
@@ -25,6 +24,8 @@
 define('PASSWORD_CRYPT_ERROR', 1);
 define('PASSWORD_ERROR', 2);
 define('PASSWORD_CONNECT_ERROR', 3);
+define('PASSWORD_IN_HISTORY', 4);
+define('PASSWORD_CONSTRAINT_VIOLATION', 5);
 define('PASSWORD_SUCCESS', 0);
 
 /**
@@ -94,6 +95,17 @@ class password extends rcube_plugin
         if (rcube_utils::get_input_value('_first', rcube_utils::INPUT_GET)) {
             $rcmail->output->command('display_message', $this->gettext('firstloginchange'), 'notice');
         }
+        else if (!empty($_SESSION['password_expires'])) {
+            if ($_SESSION['password_expires'] == 1) {
+                $rcmail->output->command('display_message', $this->gettext('passwdexpired'), 'error');
+            }
+            else {
+                $rcmail->output->command('display_message', $this->gettext(array(
+                        'name' => 'passwdexpirewarning',
+                        'vars' => array('expirationdatetime' => $_SESSION['password_expires'])
+                    )), 'warning');
+            }
+        }
 
         $rcmail->output->send('plugin');
     }
@@ -152,9 +164,9 @@ class password extends rcube_plugin
             else if ($check_strength && (!preg_match("/[0-9]/", $newpwd) || !preg_match("/[^A-Za-z0-9]/", $newpwd))) {
                 $rcmail->output->command('display_message', $this->gettext('passwordweak'), 'error');
             }
-            // password is the same as the old one, do nothing, return success
+            // password is the same as the old one, warn user, return error
             else if ($sespwd == $newpwd && !$rcmail->config->get('password_force_save')) {
-                $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
+                $rcmail->output->command('display_message', $this->gettext('samepasswd'), 'error');
             }
             // try to save the password
             else if (!($res = $this->_save($curpwd, $newpwd))) {
@@ -172,6 +184,9 @@ class password extends rcube_plugin
                     rcube::write_log('password', sprintf('Password changed for user %s (ID: %d) from %s',
                         $rcmail->get_user_name(), $rcmail->user->ID, rcube_utils::remote_ip()));
                 }
+
+                // Remove expiration date/time
+                $rcmail->session->remove('password_expires');
             }
             else {
                 $rcmail->output->command('display_message', $res, 'error');
@@ -268,11 +283,12 @@ class password extends rcube_plugin
                 'class'   => 'button mainaction',
                 'label'   => 'save',
         ));
+        $form_buttons = html::p(array('class' => 'formbuttons'), $submit_button);
 
         $out = html::div(array('class' => 'box'),
             html::div(array('id' => 'prefs-title', 'class' => 'boxtitle'), $this->gettext('changepasswd'))
             . html::div(array('class' => 'boxcontent'),
-                $disabled_msg . $table->show() . $rules . html::p(null, $submit_button)));
+                $disabled_msg . $table->show() . $rules . $form_buttons));
 
         $rcmail->output->add_gui_object('passform', 'password-form');
 
@@ -333,6 +349,12 @@ class password extends rcube_plugin
             case PASSWORD_CONNECT_ERROR:
                 $reason = $this->gettext('connecterror');
                 break;
+            case PASSWORD_IN_HISTORY:
+                $reason = $this->gettext('passwdinhistory');
+                break;
+            case PASSWORD_CONSTRAINT_VIOLATION:
+                $reason = $this->gettext('passwdconstraintviolation');
+                break;
             case PASSWORD_ERROR:
             default:
                 $reason = $this->gettext('internalerror');
@@ -369,7 +391,7 @@ class password extends rcube_plugin
 
         // Host exceptions
         $hosts = $rcmail->config->get('password_hosts');
-        if (!empty($hosts) && !in_array($_SESSION['storage_host'], $hosts)) {
+        if (!empty($hosts) && !in_array($_SESSION['storage_host'], (array) $hosts)) {
             return false;
         }
 
@@ -427,19 +449,19 @@ class password extends rcube_plugin
         switch ($method) {
         case 'des':
         case 'des-crypt':
-            $crypted = crypt($password, self::random_salt(2));
+            $crypted = crypt($password, rcube_utils::random_bytes(2));
             $prefix  = '{CRYPT}';
             break;
 
         case 'ext_des': // for BC
         case 'ext-des-crypt':
-            $crypted = crypt($password, '_' . self::random_salt(8));
+            $crypted = crypt($password, '_' . rcube_utils::random_bytes(8));
             $prefix  = '{CRYPT}';
             break;
 
         case 'md5crypt': // for BC
         case 'md5-crypt':
-            $crypted = crypt($password, '$1$' . self::random_salt(9));
+            $crypted = crypt($password, '$1$' . rcube_utils::random_bytes(9));
             $prefix  = '{CRYPT}';
             break;
 
@@ -451,7 +473,7 @@ class password extends rcube_plugin
                 $prefix .= 'rounds=' . $rounds . '$';
             }
 
-            $crypted = crypt($password, $prefix . self::random_salt(16));
+            $crypted = crypt($password, $prefix . rcube_utils::random_bytes(16));
             $prefix  = '{CRYPT}';
             break;
 
@@ -463,7 +485,7 @@ class password extends rcube_plugin
                 $prefix .= 'rounds=' . $rounds . '$';
             }
 
-            $crypted = crypt($password, $prefix . self::random_salt(16));
+            $crypted = crypt($password, $prefix . rcube_utils::random_bytes(16));
             $prefix  = '{CRYPT}';
             break;
 
@@ -473,7 +495,7 @@ class password extends rcube_plugin
             $cost   = $cost < 4 || $cost > 31 ? 12 : $cost;
             $prefix = sprintf('$2a$%02d$', $cost);
 
-            $crypted = crypt($password, $prefix . self::random_salt(22));
+            $crypted = crypt($password, $prefix . rcube_utils::random_bytes(22));
             $prefix  = '{CRYPT}';
             break;
 
@@ -504,7 +526,7 @@ class password extends rcube_plugin
             break;
 
         case 'ssha':
-            $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+            $salt = rcube_utils::random_bytes(8);
 
             if (function_exists('mhash') && function_exists('mhash_keygen_s2k')) {
                 $salt    = mhash_keygen_s2k(MHASH_SHA1, $password, $salt, 4);
@@ -530,7 +552,7 @@ class password extends rcube_plugin
             break;
 
         case 'smd5':
-            $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+            $salt = rcube_utils::random_bytes(8);
 
             if (function_exists('mhash') && function_exists('mhash_keygen_s2k')) {
                 $salt    = mhash_keygen_s2k(MHASH_MD5, $password, $salt, 4);
@@ -580,38 +602,37 @@ class password extends rcube_plugin
                 $method = 'CRAM-MD5';
             }
 
-            // use common temp dir
-            $tmp_dir = $rcmail->config->get('temp_dir');
-            $tmpfile = tempnam($tmp_dir, 'roundcube-');
+            $spec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('file', '/dev/null', 'a'));
+            $pipe = proc_open("$dovecotpw -s '$method'", $spec, $pipes);
 
-            $pipe = popen("$dovecotpw -s '$method' > '$tmpfile'", "w");
-            if (!$pipe) {
-                unlink($tmpfile);
+            if (!is_resource($pipe)) {
                 return false;
             }
-            else {
-                fwrite($pipe, $password . "\n", 1+strlen($password)); usleep(1000);
-                fwrite($pipe, $password . "\n", 1+strlen($password));
-                pclose($pipe);
 
-                $crypted = trim(file_get_contents($tmpfile), "\n");
-                unlink($tmpfile);
+            fwrite($pipes[0], $password . "\n", 1+strlen($password));
+            usleep(1000);
+            fwrite($pipes[0], $password . "\n", 1+strlen($password));
 
-                if (!preg_match('/^\{' . $method . '\}/', $crypted)) {
-                    return false;
-                }
+            $crypted = trim(stream_get_contents($pipes[1]), "\n");
 
-                if (!$default) {
-                    $prefixed = (bool) $rcmail->config->get('password_dovecotpw_with_method');
-                }
+            fclose($pipes[0]);
+            fclose($pipes[1]);
+            proc_close($pipe);
 
-                if (!$prefixed) {
-                    $crypted = trim(str_replace('{' . $method . '}', '', $crypted));
-                }
-
-                $prefixed = false;
+            if (!preg_match('/^\{' . $method . '\}/', $crypted)) {
+                return false;
             }
 
+            if (!$default) {
+                $prefixed = (bool) $rcmail->config->get('password_dovecotpw_with_method');
+            }
+
+            if (!$prefixed) {
+                $crypted = trim(str_replace('{' . $method . '}', '', $crypted));
+            }
+
+            $prefixed = false;
+
             break;
 
         case 'hash': // deprecated
@@ -653,22 +674,4 @@ class password extends rcube_plugin
 
         return $crypted;
     }
-
-    /**
-     * Used to generate a random salt for crypt-style passwords
-     *
-     * Code originaly from the phpLDAPadmin development team
-     * http://phpldapadmin.sourceforge.net/
-     */
-    static function random_salt($length)
-    {
-        $possible = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ./';
-        $str      = '';
-
-        while (strlen($str) < $length) {
-            $str .= substr($possible, (rand() % strlen($possible)), 1);
-        }
-
-        return $str;
-    }
 }
